#!/usr/bin/perl -w

=head1 NAME

Debconf::FrontEnd::Gnome - GUI Gnome frontend

=cut

package Debconf::FrontEnd::Gnome;
use strict;
use utf8;
use Debconf::Gettext;
use Debconf::Config;
use Debconf::Encoding qw(to_Unicode);
use base qw{Debconf::FrontEnd};
use Config::IniFiles;
use diagnostics;

# Catch this so as not to confuse the poor users if Gtk or Gnome are not
# installed.
eval q{
	use Gtk2;
};
die "Unable to load Gtk -- is libgtk2-perl installed?\n" if $@;

=head1 DESCRIPTION

This FrontEnd is a Gnome UI for Debconf.

=head1 METHODS

=over 4

=item init

Set up most of the GUI.

=cut

our @ARGV_for_gnome=('--sm-disable');

sub create_assistant_page {
	my $this=shift;

	my $align = Gtk2::Alignment->new(0, 0.5, 1, 0);
	$this->assistant_page(Gtk2::VBox->new);
	$align->add($this->assistant_page);
	$this->assistant->append_page($align);

	if ($this->logo) {
		$this->app_icon->set_from_pixbuf($this->logo);
	}

	$this->configure_assistant_page;
	$align->show_all;
}

sub desktopfile_for_package {
	my $package=shift;

	foreach (split(/\n/, `dpkg -L $package`)) {
		if ($_ =~ /.desktop$/) {
			return $_;
		}
	}
}

sub desktop_entry {
	my $this=shift;
	my $desktopfile=shift;

	my $icon = '';
	my $name = '';
	if (length $desktopfile) {
		my $cfg = Config::IniFiles->new( -file => $desktopfile );
		$icon = $cfg->val('Desktop Entry', 'Icon');
		$name = $cfg->val('Desktop Entry', 'Name');
	}
	my $theme = Gtk2::IconTheme->get_default();
	if (length $icon) {
		$this->logo($theme->load_icon ($icon, 48, 'GTK_ICON_LOOKUP_USE_BUILTIN'));
	} else {
		$this->logo($theme->load_icon ('distributor-logo', 48, 'GTK_ICON_LOOKUP_USE_BUILTIN'));
	}
	$this->app_icon->set_from_pixbuf($this->logo);

	if (length $name) {
		$this->dialog_header->set_markup("<big>" . sprintf($this->dialog_header_label, $name) . "</big>");
	}
}

sub default_title {
	# Override the default_title function to capture the package name.
	my $this=shift;
	my $title=shift;
	$this->desktop_entry(desktopfile_for_package($title));

	if (not length $this->dialog_header->get_label()) {
		$this->dialog_header->set_markup("<big>" . sprintf($this->dialog_header_label, $title) . "</big>");
	}
}
sub configure_assistant_page {
	my $this=shift;

	if ($this->capb_backup) {
		push($this->page_can_back_up, 1);
	} else {
		push($this->page_can_back_up, 0);
	}
}

sub reset_assistant_page {
	my $this=shift;

	$this->assistant_page($this->assistant->get_nth_page($this->assistant->get_current_page)->get_child);
	foreach my $element ($this->assistant_page->get_children) {
		$this->assistant_page->remove($element);
	}
}

my $prev_page = 0;

# this gets called on clicking next/previous buttons
sub prepare_callback {
	my $this=shift;
	my $current_page = $this->assistant->get_current_page;
	
	if (${$this->page_can_back_up}[$current_page]) {
		$this->prev_page->show;
	} else {
		$this->prev_page->hide;
	}

	if ($prev_page < $current_page) {
		$this->goback(0);
		if (Gtk2->main_level) {
			Gtk2->main_quit;
		}
	} elsif ($prev_page > $current_page) {
		$this->goback(1);
		if (Gtk2->main_level) {
			Gtk2->main_quit;
		}
	}
	$prev_page = $current_page;
}

sub forward_page_func {
	my ($current_page, $assistant) = @_;

	if ($current_page == $assistant->get_n_pages - 1) {
		return 0;
	} else {
		return $current_page + 1;
	}
}

sub init {
	my $this=shift;
	
	# Ya know, this really sucks. The authors of GTK seemed to just not
	# conceive of a program that can, *gasp*, work even if GTK doesn't
	# load. So this thing throws a fatal, essentially untrappable
	# error. Yeesh. Look how far I must go out of my way to make sure
	# it's not going to destroy debconf..
	if (fork) {
		wait(); # for child
		if ($? != 0) {
			die "DISPLAY problem?\n";
		}
	}
	else {
		@ARGV=@ARGV_for_gnome; # temporary change at first
		Gtk2->init;
		exit(0); # success
	}
	
	my @gnome_sucks=@ARGV;
	@ARGV=@ARGV_for_gnome;
	Gtk2->init;
	@ARGV=@gnome_sucks;
	
	$this->SUPER::init(@_);
	$this->interactive(1);
	$this->capb('backup');
	$this->need_tty(0);
	
	$this->dialog_header_label(gettext("%s has a problem and needs your help to finish installing."));
	$this->page_can_back_up([]);

	$this->dialog(Gtk2::Window->new);
	$this->dialog->set_border_width(20);
	my $vbox = Gtk2::VBox->new;
	$vbox->set_spacing(6);
	$this->dialog->add($vbox);
	$this->assistant(Gtk2::Notebook->new);
	$this->prev_page(Gtk2::Button->new_with_label(gettext("Go Back")));
	$this->next_page(Gtk2::Button->new_with_label(gettext("Continue")));
	my $buttonbox = Gtk2::HButtonBox->new;
	$buttonbox->set_layout_default("GTK_BUTTONBOX_END");
	$buttonbox->set_spacing(6);
	$buttonbox->add($this->prev_page);
	$buttonbox->add($this->next_page);
	my $hbox = Gtk2::HBox->new();
	$hbox->set_spacing(12);
	$this->app_icon(Gtk2::Image->new);
	$this->app_icon->set_alignment(0.5, 0);
	$hbox->pack_start($this->app_icon, 0, 0, 0);
	$vbox->pack_start($hbox, 1, 1, 0);
	$vbox->pack_start($buttonbox, 0, 0, 0);
	my $vbox2 = Gtk2::VBox->new();
	$vbox2->set_spacing(6);
	$this->dialog_header(Gtk2::Label->new);
	$this->dialog_header->set_alignment(0, 0.5);
	$this->dialog_header->set_line_wrap(1);
	$this->report(Gtk2::CheckButton->new_with_label(gettext("Send an error report to help fix this problem")));
	$vbox2->pack_start($this->dialog_header, 0, 0, 0);
	$vbox2->pack_start($this->assistant, 1, 1, 0);
	$vbox2->pack_start($this->report, 0, 0, 0);
	$hbox->pack_start($vbox2, 1, 1, 0);

	$this->dialog->set_position("center");
	my $hostname = `hostname`;
	chomp $hostname;
	$this->assistant->signal_connect("delete_event", sub { exit 1 });
	$this->assistant->set_show_tabs(0);
	$this->assistant->set_show_border(0);

	my $distribution='';
	if (system('type lsb_release >/dev/null 2>&1') == 0) {
		$distribution=`lsb_release -is`;
		chomp $distribution;
	} elsif (-e '/etc/debian_version') {
		$distribution='debian';
	}
	$this->dialog->set_title(to_Unicode($distribution));
	
	$this->dialog->signal_connect("destroy", sub { exit 1});
	$this->next_page->signal_connect("clicked", sub { $this->assistant->next_page; $this->prepare_callback; });
	$this->prev_page->signal_connect("clicked", sub { $this->assistant->prev_page; $this->prepare_callback; });
	$this->create_assistant_page();

	# hide cancel and the close button to avoid bugreports from
	# people that force to close the window. Also hide resize buttons, which
	# create an obscene amount of empty space.
	$this->assistant->signal_connect("realize", sub { 
		$this->assistant->window->set_functions(["move"]) 
	});

	$this->dialog->show_all;
	$this->prev_page->hide;
}

=item go

Creates and lays out all the necessary widgets, then runs them to get
input.

=cut

sub go {
	my $this=shift;
	my @elements=@{$this->elements};

	$this->reset_assistant_page;

	my $interactive='';
	foreach my $element (@elements) {
		# Noninteractive elements have no hboxes.
		next unless $element->vbox;

		$interactive=1;
		$this->assistant_page->pack_start($element->vbox, $element->fill, $element->expand, 0);
	}

	if ($interactive) {
		$this->configure_assistant_page;
		if ($this->assistant->get_current_page == $this->assistant->get_n_pages - 1) {
			# Create the next page so that GtkAssistant doesn't
			# hide the Forward button.
			$this->create_assistant_page();
		} 
		Gtk2->main;
	}

	# Display all elements. This does nothing for gnome
	# elements, but it causes noninteractive elements to do
	# their thing.
	foreach my $element (@elements) {
		$element->show;
	}

	return '' if $this->goback;
	return 1;
}

sub progress_start {
	my $this=shift;
	$this->SUPER::progress_start(@_);

	$this->reset_assistant_page;

	my $element=$this->progress_bar;
	$this->assistant_page->pack_start($element->vbox, $element->fill, $element->expand, 0);
	# TODO: no backup support yet
	$this->configure_assistant_page;
	$this->assistant->set_page_complete($this->assistant_page, 0);
	$this->assistant->show_all;

	while (Gtk2->events_pending) {
		Gtk2->main_iteration;
	}
}

sub progress_set {
	my $this=shift;

	my $ret=$this->SUPER::progress_set(@_);

	while (Gtk2->events_pending) {
		Gtk2->main_iteration;
	}

	return $ret;
}

sub progress_info {
	my $this=shift;
	my $ret=$this->SUPER::progress_info(@_);

	while (Gtk2->events_pending) {
		Gtk2->main_iteration;
	}

	return $ret;
}

sub progress_stop {
	my $this=shift;
	$this->SUPER::progress_stop(@_);

	while (Gtk2->events_pending) {
		Gtk2->main_iteration;
	}

	if ($this->assistant->get_current_page == $this->assistant->get_n_pages - 1) {
		$this->create_assistant_page();
	}
	# automatically go to the next page now
	$this->assistant->set_current_page($prev_page + 1);
}

=back

=head1 AUTHOR

Eric Gillespie <epg@debian.org>
Gustavo Noronha Silva <kov@debian.org>
Evan Dandrea <ev@ubuntu.com>

=cut

1
